package com.geetest.gateauth.demo.ui.onelogin;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.graphics.Color;
import android.graphics.Typeface;
import android.graphics.drawable.ColorDrawable;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.util.DisplayMetrics;
import android.util.Log;
import android.util.TypedValue;
import android.view.Display;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.geetest.gateauth.demo.R;
import com.geetest.gateauth.demo.SuccessActivity;
import com.geetest.gateauth.demo.util.Constants;
import com.geetest.gateauth.demo.util.DensityUtils;
import com.geetest.gateauth.demo.util.HttpUtils;
import com.geetest.gateauth.demo.util.Utils;
import com.geetest.onelogin.OneLoginHelper;
import com.geetest.onelogin.config.AuthRegisterViewConfig;
import com.geetest.onelogin.config.OneLoginThemeConfig;
import com.geetest.onelogin.config.ProtocolShakeStyle;
import com.geetest.onelogin.config.UserInterfaceStyle;
import com.geetest.onelogin.listener.AbstractOneLoginListener;
import com.geetest.onelogin.view.GTContainerWithLifecycle;
import com.geetest.onelogin.view.GTGifView;
import com.geetest.onelogin.view.GTVideoView;
import com.geetest.onelogin.view.LoadingImageView;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;

import static com.geetest.gateauth.demo.util.Constants.ONELOGIN_DIALOG_MODE;
import static com.geetest.gateauth.demo.util.Constants.ONELOGIN_FLOAT_MODE;
import static com.geetest.gateauth.demo.util.Constants.ONELOGIN_FULLSCREEN_MODE;
import static com.geetest.gateauth.demo.util.Constants.ONELOGIN_LANDSCAPE_MODE;
import static com.geetest.gateauth.demo.util.Constants.TAG;

public class OneLoginFragment extends Fragment implements View.OnClickListener {

    /**
     * 预取号loading 对话框
     */
    private Dialog preLoadingDialog;
    /**
     * 一键登录取号时是否使用自定义 loading 对话框
     */
    private final boolean useCustomLoading = true;
    private Handler backHandler;
    private final Handler mainHandler = new Handler(Looper.getMainLooper());

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_onelogin, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        TextView bigTv = view.findViewById(R.id.onelogin_select_bigtv_tv);
        ImageView floatImage = view.findViewById(R.id.onelogin_select_float_iv);
        ImageView popImage = view.findViewById(R.id.onelogin_select_pop_iv);
        ImageView fullScreenImage = view.findViewById(R.id.onelogin_select_fullscreen_iv);
        ImageView landscapeImage = view.findViewById(R.id.onelogin_select_landscape_iv);

        bigTv.getPaint().setFakeBoldText(true);

        changeAlpha(floatImage);
        changeAlpha(popImage);
        changeAlpha(fullScreenImage);
        changeAlpha(landscapeImage);

        floatImage.setOnClickListener(this);
        popImage.setOnClickListener(this);
        fullScreenImage.setOnClickListener(this);
        landscapeImage.setOnClickListener(this);

        HandlerThread handlerThread = new HandlerThread("oneLogin-demo");
        handlerThread.start();
        backHandler = new Handler(handlerThread.getLooper());
    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        int mode = ONELOGIN_FULLSCREEN_MODE; //竖屏全屏
        if (id == R.id.onelogin_select_float_iv) { //底部弹窗
            mode = ONELOGIN_FLOAT_MODE;
        } else if (id == R.id.onelogin_select_pop_iv) { //居中弹窗
            mode = ONELOGIN_DIALOG_MODE;
        } else if (id == R.id.onelogin_select_landscape_iv) { //横屏模式
            mode = ONELOGIN_LANDSCAPE_MODE;
        }
        login(mode);
    }

    @SuppressLint("ClickableViewAccessibility")
    public void changeAlpha(View view) {
        view.setOnTouchListener((v, motionEvent) -> {
            int action = motionEvent.getAction();
            if (action == MotionEvent.ACTION_DOWN) {
                view.setAlpha(0.7f);
            } else if (action == MotionEvent.ACTION_UP) {
                view.setAlpha(1f);
            }
            return false;
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getActivity().getRequestedOrientation() == ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE) {
            getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        dismissPreLoading();
    }

    @Override
    public void onStop() {
        super.onStop();
        dismissPreLoading();
    }

    private void showPreLoading() {
        if (preLoadingDialog == null) {
            preLoadingDialog = new ProgressDialog(getActivity());
            preLoadingDialog.setTitle("正在提取本机号码");
        }
        if (!preLoadingDialog.isShowing()) {
            preLoadingDialog.show();
        }
    }

    private void dismissPreLoading() {
        if (preLoadingDialog != null && preLoadingDialog.isShowing()) {
            preLoadingDialog.dismiss();
        }
    }

    /**
     * 调用sdk发起一键登录
     * @param mode 展示的四种不同授权页的场景
     */
    public void login(int mode) {
        //预取号无效时，可先加载自定义 loading 对话框，然后调用 requestToken，
        //SDK 内部会进行自动预取号，待授权页拉起后(当前 Activity onPause)即可关闭 loading 对话框
        if (!OneLoginHelper.with().isPreGetTokenResultValidate()) {
            showPreLoading();
        }
        if (mode == ONELOGIN_FULLSCREEN_MODE) {
            //三方登录自定义 UI 配置，可选配
            initThirdLogin();
        } else if (mode == ONELOGIN_LANDSCAPE_MODE) {
            // 需要使用横屏模式的，只要调用requestToken时应用所处的界面是横屏方向即可
            // 另外，OneLoginThemeConfig需要设置横屏的布局参数，比如控件距离顶部的距离不能超过横屏的高度
            if (getActivity().getRequestedOrientation() == ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) {
                getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
            }
        }
        OneLoginHelper.with().requestToken(generateUiConfig(mode),
                new IDOneLoginListener(this, mode == ONELOGIN_FLOAT_MODE)); //浮窗模式增加气泡提示框效果演示
    }

    /**
     * 自定义的第三方登录设置
     */
    private void initThirdLogin() {
        Context context = getContext();
        LayoutInflater inflater1 = LayoutInflater.from(context);
        RelativeLayout relativeLayout = (RelativeLayout) inflater1.inflate(R.layout.layout_other_login, null);
        RelativeLayout.LayoutParams layoutParamsOther = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
        layoutParamsOther.setMargins(0, DensityUtils.dip2px(context, 430), 0, 0);
        layoutParamsOther.addRule(RelativeLayout.CENTER_HORIZONTAL);
        layoutParamsOther.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        layoutParamsOther.setMargins(0,0,0,DensityUtils.dip2px(context,24));
        relativeLayout.setLayoutParams(layoutParamsOther);
        relativeLayout.findViewById(R.id.weixin).setOnClickListener(v -> {
            Utils.toastMessage(context, "微信登录");
            // 自定义的第三方登录
            // context 为一键登录授权页 的 Context(即 OneLoginActivity 的 Context)，注意判空
            // oneLoginActivity.startActivityForResult(new Intent(oneLoginActivity, ThirdLoginActivity.class), Constants.THIRD_LOGIN_CODE);
        });
        relativeLayout.findViewById(R.id.qq).setOnClickListener(v -> {
            Utils.toastMessage(context, "qq登录");
        });
        relativeLayout.findViewById(R.id.weibo).setOnClickListener(v -> {
            Utils.toastMessage(context, "微博登录");
        });
        /**
         * 添加自定义控件，addOneLoginRegisterViewConfig 调用多次可添加多个自定义控件，用不同的 id 区分
         * 注意添加的控件不要覆盖下面原有的控件，不然可能导致无法点击的问题
         */
        OneLoginHelper.with().addOneLoginRegisterViewConfig("third_login_button", new AuthRegisterViewConfig.Builder()
                .setView(relativeLayout)
                .setRootViewId(AuthRegisterViewConfig.RootViewId.ROOT_VIEW_ID_BODY)
                .setCustomInterface(context1 -> {
                    Utils.toastMessage(context1, "动态注册的其他按钮");
                })
                .build()
        );
    }

    /**
     * 添加自定义控件，供参考
     * 注意：横屏的自定义控件的边距需要自己再重新按照设计图设计
     * Demo中自定义控件只支持竖屏，横屏的自定义控件需要自己实现
     */
    private void initView() {
        Button mTitleBtn = new Button(getActivity());
        mTitleBtn.setText("其他");
        mTitleBtn.setTextColor(0xff000000);
        mTitleBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14);
        mTitleBtn.setBackgroundColor(Color.TRANSPARENT);
        RelativeLayout.LayoutParams mLayoutParams = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
        mLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT, RelativeLayout.TRUE);
        mTitleBtn.setLayoutParams(mLayoutParams);


        Button mBtn = new Button(getActivity());
        mBtn.setText("其他方式登录");
        mBtn.setTextColor(0xff3a404c);
        mBtn.setBackgroundColor(Color.TRANSPARENT);
        mBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13);
        RelativeLayout.LayoutParams mLayoutParams1 = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
        mLayoutParams1.addRule(RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.TRUE);
        mLayoutParams1.setMargins(0, DensityUtils.dip2px(getContext(), 400), 0, 0);
        mBtn.setLayoutParams(mLayoutParams1);


        OneLoginHelper.with().addOneLoginRegisterViewConfig("title_button", new AuthRegisterViewConfig.Builder()
                .setView(mTitleBtn)
                .setRootViewId(AuthRegisterViewConfig.RootViewId.ROOT_VIEW_ID_TITLE_BAR)
                .setCustomInterface(context -> {
                    Utils.toastMessage(context, "动态注册的其他按钮");
                })
                .build()
        ).addOneLoginRegisterViewConfig("other_button", new AuthRegisterViewConfig.Builder()
                .setView(mBtn)
                .setRootViewId(AuthRegisterViewConfig.RootViewId.ROOT_VIEW_ID_BODY)
                .setCustomInterface(context -> {
                    Utils.toastMessage(context, "动态注册的其他登录按钮");
                })
                .build()
        );
    }

    /**
     * 为了避免内存泄漏，建议设计为静态内部类
     */
    private static class IDOneLoginListener extends AbstractOneLoginListener {
        private final WeakReference<OneLoginFragment> fragmentReference;
        private WeakReference<Activity> authActivityWeakReference;
        private WeakReference<RelativeLayout> checkBoxWeakReference;

        /**
         * 自定义的取号 loading 对话框
         */
        private ProgressDialog loadingDialog;

        private final boolean usePopToast;

        public IDOneLoginListener(OneLoginFragment fragment, boolean usePopToast) {
            this.fragmentReference = new WeakReference<>(fragment);
            this.usePopToast = usePopToast;
        }

        /**
         * 取号成功、取号失败、切换账号、点返回或者按返回键从授权页返回取消登录等都从该回调返回结果
         * @param jsonObject
         */
        @Override
        public void onResult(final JSONObject jsonObject) {
            Log.i(TAG, "取号结果为：" + jsonObject.toString());
            OneLoginFragment fragment = fragmentReference.get();
            if (fragment != null) {
                fragment.dismissPreLoading();
            }
            try {
                int status = jsonObject.getInt("status");
                //status=200 表示取号成功，可携带 process_id、token、authcode 去服务端换取手机号
                if (status == Constants.SUCCESS_CODE) {
                    final String process_id = jsonObject.getString("process_id");
                    final String token = jsonObject.getString("token");
                    /**
                     * authcode值在电信卡通过 token 换取手机号时必要参数，为了避免服务端校验出错，尽量三网都传
                     */
                    final String authCode = jsonObject.optString("authcode");
                    if (fragment != null) {
                        fragment.backHandler.post(() -> verify(process_id, token, authCode));
                    }
                    return;
                } else {
                    //在页面返回失败的情况下 如果不注重实用返回的监听事件，可以对于返回的事件不进行处理
                    String errorCode = jsonObject.getString("errorCode");
                    if (errorCode.equals("-20301") || errorCode.equals("-20302")) {
                        onVerifyEnd();
                        Log.d(TAG, "用户点击返回键关闭了授权页面");
                        return;
                    }
                    // 用户选择切换账号登录，收到此回调后可关闭授权页，跳转或降级走其他方式登录，或者待用户选择
                    // 重新拉起授权页进行一键登录
                    if (errorCode.equals("-20303")) {
                        Log.d(TAG, "用户点击切换账号");
                    }
                    //其他错误码产生原因与处理方式，请参考部署文档中的错误码说明
                    if (fragment != null) {
                        Utils.toastMessage(fragment.getContext(), "取号失败:" + jsonObject.toString());
                    }
                }
            } catch (JSONException e) {
                if (fragment != null) {
                    Utils.toastMessage(fragment.getContext(), "取号失败:" + jsonObject.toString());
                }
            }
            onVerifyEnd();
            OneLoginHelper.with().dismissAuthActivity();
        }

        /**
         * 授权页面点击隐私协议条款的回调
         * 如有需要自定义隐私条款页，可在此回调中跳转到自定义隐私条款页展示隐私条款
         * 自定义隐私条款页参考 setPrivacyLayout 接口说明
         * @param name 隐私条款名字
         * @param url  隐私条款URL
         */
        @Override
        public void onPrivacyClick(String name, String url) {
            Log.d(TAG, "当前点击了隐私条款名为：" + name + "---地址为:" + url);
        }

        /**
         * 授权页点击一键登录按钮的回调
         */
        @Override
        public void onLoginButtonClick() {
            Log.d(TAG, "当前点击了登录按钮");

            //如果checkbox未勾选弹出popupWindow
            boolean privacyChecked = OneLoginHelper.with().isPrivacyChecked();
            if (usePopToast && !privacyChecked) {
                RelativeLayout rvCheckBox = checkBoxWeakReference.get();
                Context context = authActivityWeakReference.get();
                if (rvCheckBox != null && context != null) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                        boolean inMultiWindowMode = ((Activity) rvCheckBox.getContext()).isInMultiWindowMode();
                        if (inMultiWindowMode) {
                            Log.i(TAG, "popup is not work well inMultiWindowMode");
                            toastUnChecked(context);
                            return;
                        }
                    }
                    LayoutInflater inflater = LayoutInflater.from(context);
                    View popupRootView = inflater.inflate(R.layout.layout_popup_toast, null);
                    PopupWindow popupWindow = new PopupWindow(popupRootView,
                            ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
                    popupWindow.setBackgroundDrawable(new ColorDrawable());
                    popupWindow.setOutsideTouchable(true);

                    //pop的内容宽高
                    popupRootView.measure(View.MeasureSpec.UNSPECIFIED,View.MeasureSpec.UNSPECIFIED);
                    int popHeight = popupRootView.getMeasuredHeight();

                    //rvCheckBox的左上角坐标
                    int[] checkBoxLocation = new int[2];
                    rvCheckBox.getLocationInWindow(checkBoxLocation);

                    int dp8 = DensityUtils.dip2px(context, 8);

                    //x y 坐标的值为以Activity左上角为原点的坐标值
                    popupWindow.showAtLocation(rvCheckBox, Gravity.TOP | Gravity.START,
                            checkBoxLocation[0] + dp8, checkBoxLocation[1] - popHeight);
                }
            }
        }

        private void toastUnChecked(Context context) {
            Toast.makeText(context, "请同意服务条款", Toast.LENGTH_SHORT).show();
        }

        /**
         * 授权页点击切换账号按钮的回调
         * 2.1.4 版本新增，同时 onResult 回调中仍然会返回 -20303 返回码的回调结果
         */
        @Override
        public void onSwitchButtonClick() {
            super.onSwitchButtonClick();
        }

        /**
         * 授权页点击标题栏返回按钮或者手机返回键的回调
         * 2.1.4 版本新增，同时 onResult 回调中仍然会返回 -20301 和 -20302 返回码的回调结果
         */
        @Override
        public void onBackButtonClick() {
            super.onBackButtonClick();
        }

        /**
         * 拉起授权页时返回脱敏手机号的回调
         * @param phone 脱敏手机号
         */
        @Override
        public void onRequestTokenSecurityPhone(String phone) {
            super.onRequestTokenSecurityPhone(phone);
        }

        /**
         * 授权页点击一键登录按钮后开始启动 loading 的回调
         * 当用户未集成第三方验证时，onLoginButtonClick 与 onLoginLoading 先后几乎同时发回
         * 当用户集成了第三方验证时，onLoginButtonClick 在点击一键登录时立即发回调，
         *  onLoginLoading 在验证结束后开始取号时发回调
         */
        @Override
        public void onLoginLoading() {
            Log.d(TAG, "开始加载 loading");
            /**
             * 如需自定义对话框式 loading，可在此处开始显示自定义 loading 对话框
             */
            OneLoginFragment loginFragment = fragmentReference.get();
            Activity authActivity = authActivityWeakReference.get();
            if (loginFragment == null || authActivity == null) {
                return;
            }
            if (loginFragment.useCustomLoading) {
                loadingDialog = ProgressDialog.show(authActivity, null, "一键登录取号中", true, true);
            }
        }

        /**
         * 拉起授权页成功时授权页 Activity 创建时的回调
         * @param activity 授权页 Activity 实例
         */
        @Override
        public void onAuthActivityCreate(Activity activity) {
            Log.d(TAG, "当前弹起授权页面:" + activity.getClass().getSimpleName());
            // 如需设置弹框模式背景蒙层透明度，可在此处设置
            //activity.getWindow().setDimAmount(0.8f);
            OneLoginFragment loginFragment = fragmentReference.get();
            if (loginFragment == null) {
                return;
            }
            loginFragment.dismissPreLoading();

            // 保存当前授权页 Activity 实例，可用于弹出自定义弹框。
            // 如果用授权页之前的 Activity 作为 Context，弹出的对话框会被授权页遮盖
            authActivityWeakReference = new WeakReference<>(activity);
        }

        /**
         * 一键登录授权页跳转第三方登录后的 onActivityResult 回调
         * OneLoginActivity 覆写(Override) 了 onActivityResult 回调，
         *  并通过 onAuthActivityResult 回调透传全部参数
         *
         * @param requestCode 请求码
         * @param resultCode  结果码
         * @param data        返回数据
         */
        @Override
        public void onAuthActivityResult(int requestCode, int resultCode, Intent data) {
            super.onAuthActivityResult(requestCode, resultCode, data);
            // 解析返回参数，处理登录结果
            // 如成功则关闭一键登录授权页，更新登录状态，或者跳转登录成功后的页面
            // 失败时，可继续停留在一键登录授权页面，可继续登录
        }

        /**
         * 用户点击授权页隐私条款项跳转到隐私页显示隐私条款内容，隐私页 Activity 创建时的回调
         * 用户若自定义了隐私页，就不会受到该回调
         * @param activity 隐私条款页 Activity 实例
         */
        @Override
        public void onAuthWebActivityCreate(Activity activity) {
            Log.d(TAG, "当前弹起授权Web页面:" + activity.getClass().getSimpleName());
        }

        /**
         * 授权页隐私栏选择框点击事件回调
         * @param isChecked 选择框当前的选择状态
         */
        @Override
        public void onPrivacyCheckBoxClick(boolean isChecked) {
            Log.d(TAG, "当前点击了CheckBox, CheckBox 选择状态:" + isChecked);
        }

        @Override
        public void onAuthActivityViewInit(TextView tvTitle, TextView tvNumber, TextView tvSwitchAccount,
                                           TextView tvLogin, TextView tvSlogan, TextView tvPrivacy,
                                           CheckBox checkBox, RelativeLayout rvCheckBox,
                                           RelativeLayout rvTitle, RelativeLayout rvSecondRoot,
                                           RelativeLayout rvMainLayout, LinearLayout lvPrivacy,
                                           RelativeLayout rvLogin, RelativeLayout rvSwitchAccount,
                                           ImageView ivBack, ImageView ivLogo, LoadingImageView ivLoading,
                                           GTGifView gifLoading, GTContainerWithLifecycle bgContainer,
                                           GTVideoView vdoViBackground, ImageView imageBackground,
                                           FrameLayout flRoot) {
            checkBoxWeakReference = new WeakReference<>(rvCheckBox);
        }

        /**
         * 手机号校验接口 需要网站主配置相关接口进行获取手机号等操作
         * 在这个阶段，也需要调用 OneLoginHelper.with().dismissAuthActivity() 来进行授权页的销毁
         * 注意: 0.8.0 之后的版本 loading 由用户自己控制消失时间
         */
        private void verify(String id, String token, String authCode) {
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("process_id", id);
                jsonObject.put("token", token);
                jsonObject.put("authcode", authCode);
                /**
                 * 注意，此处的id_2_sign参数是<url>https://onepass.geetest.com/onelogin/result</url>服务端 demo 接口所需要的参数
                 * 具体的参数与网站主服务端有关
                 */
                jsonObject.put("id_2_sign", Constants.APP_ID_OL);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            final String result = HttpUtils.requestNetwork(Constants.CHECK_PHONE_URL, jsonObject);
            Log.i(TAG, "校验结果为:" + result);
            OneLoginFragment loginFragment = fragmentReference.get();
            if (loginFragment != null) {
                loginFragment.mainHandler.post(() -> {
                    /**
                     * 关闭 loading 动画
                     */
                    OneLoginHelper.with().stopLoading();

                    /**
                     * 关闭授权页面
                     * sdk内部除了返回或初始化异常以外是不关闭授权页面的，需要开发者自行关闭
                     */
                    OneLoginHelper.with().dismissAuthActivity();
                    try {
                        JSONObject jsonObject1 = new JSONObject(result);
                        int status = jsonObject1.getInt("status");
                        if (status == Constants.SUCCESS_CODE) {
                            onVerifyEnd();
                            Intent intent = new Intent(loginFragment.getActivity(), SuccessActivity.class);
                            intent.putExtra(Constants.IS_ONELOGIN_SUCCEED, true);
                            loginFragment.startActivity(intent);
                            return;
                        } else {
                            Utils.toastMessage(loginFragment.getContext(), "校验失败");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    onVerifyEnd();
                });
            }
        }

        /**
         * 一键登录结束
         */
        private void onVerifyEnd() {
            OneLoginFragment loginFragment = fragmentReference.get();
            if (loginFragment == null) {
                return;
            }
            if (loginFragment.useCustomLoading) {
                if (loadingDialog != null) {
                    loadingDialog.dismiss();
                    loadingDialog = null;
                }
            }
        }
    };


    /**
     * 配置页面布局(默认竖屏)
     * 注意事项：
     * 1. 配置项的图片，地址都应为`drawable`目录下资源文件名称，支持配置静态 png/jpg 图片，或者带状态选择的 xml 文件
     * 2. OffsetY 与 OffsetY_B 两者必须有一个值为 0,偏移量以不为 0 的方向作为基准。
     * 3. 颜色为int类型，尽量使用res中的colorId，例如 R.color.colorPrimary。也可以使用0xFFFFFFFF。
     *    sdk会优先按照colorId去查找资源，若查找不到则按照固定颜色值使用，传0会认为无效值，将使用默认颜色
     *
     * @return OneLoginThemeConfig
     */
    private OneLoginThemeConfig generateUiConfig(int mode) {
        int screenWidth = getResources().getDisplayMetrics().widthPixels;
        int screenHeight = getResources().getDisplayMetrics().heightPixels;
        Log.i(TAG, "display width:" + screenWidth + " height:" + screenHeight);
        int width = DensityUtils.px2dip(this.getContext(), screenWidth);
        int navigationHeight = DensityUtils.px2dip(this.getContext(), getNavigationBarHeight());

        switch (mode) {
            case ONELOGIN_FLOAT_MODE:
                return new OneLoginThemeConfig.Builder()
                        .setDialogTheme(true, width, 500, 0, 0, true, true)
                        .setAuthBGImgPath("bg_dialog")
                        .setNavigationBar(Color.BLACK, UserInterfaceStyle.UNSPECIFIED, false)
                        .setAuthNavLayout(R.color.colorPrimary, 49, true, false)
                        .setAuthNavTextView("一键登录", 0xFFFFFFFF, 17, false, "服务条款", 0xFF000000, 17)
                        .setAuthNavReturnImgView("gt_one_login_ic_chevron_left_black", 40, 40, false, 8)
                        .setLogoImgView("gt_one_login_logo", 71, 71, false, 125 - navigationHeight, 0, 0)
                        .setSloganView(R.color.GOLColorGray, 10, 370 - navigationHeight, 0, 0)
                        .setNumberView(R.color.GOLColorNumberText, 24, 200 - navigationHeight, 0, 0)
                        .setSwitchView("切换账号", R.color.colorPrimary, 14, false, 249 - navigationHeight, 0, 0)
                        .setLogBtnLayout("gt_one_login_btn", "gt_one_login_btn_unchecked",290, 45, 310 - navigationHeight, 0, 0)
                        .setLogBtnTextView("一键登录", 0xFFFFFFFF, 18)
                        .setLogBtnLoadingView(useCustomLoading ? "" : "umcsdk_load_dot_white", 20, 20, 12)
                        .setPrivacyUnCheckedToastText(false, "") //可以关闭SDK提供的toast提示，在点击授权的回调中自定义提示
                        .setPrivacyCheckBox("gt_one_login_unchecked", "gt_one_login_checked", false, 24, 24, 2)
                        .setPrivacyLayout(256, 0, 18, 0, true)
                        .setPrivacyClauseView(R.color.GOLColorGray, R.color.colorPrimary, 12)
                        .setPrivacyTextView("登录即同意", "和", "、", "并使用本机号码登录")
                        .build();
            case ONELOGIN_DIALOG_MODE:
                return new OneLoginThemeConfig.Builder()
                        .setDialogTheme(true, 314, 465, 0, 0, false, true)
                        .setAuthBGImgPath("bg_dialog")
                        .setAuthNavReturnImgView("gt_one_login_ic_chevron_left_black", 40, 40, false, 8)
                        .setLogoImgView("gt_one_login_logo", 71, 71, false, 60, 0, 0)
                        .setSloganView(R.color.GOLColorGray, 10, 270, 0, 0)
                        .setNumberView(R.color.GOLColorNumberText, 24, 125, 0, 0)
                        .setSwitchView("切换账号", R.color.colorPrimary, 14, false, 165, 0, 0)
                        .setLogBtnLayout("gt_one_login_btn", "gt_one_login_btn_unchecked",268, 45, 220, 0, 0)
                        .setLogBtnTextView("一键登录", 0xFFFFFFFF, 18)
                        .setLogBtnLoadingView(useCustomLoading ? "" : "umcsdk_load_dot_white", 20, 20, 12)
                        .setPrivacyLayout(256, 0, 1, 0, true)
                        .build();
            case ONELOGIN_FULLSCREEN_MODE:
                return new OneLoginThemeConfig.Builder()
                        .setStatusBar(0, UserInterfaceStyle.UNSPECIFIED, true)
                        .setAuthNavReturnImgView("gt_one_login_ic_chevron_left_black", 40, 40, false, 8)
                        // 一键登录背景支持配置带状态选择器 selector 的 drawable，配合 setLogBtnDisableIfUnChecked 接口可实现登录按钮状态随选择框选中状态切换
                        .setLogBtnLayout("gt_one_login_btn", "gt_one_login_btn_unchecked", 290, 45, 310, 0, 0)
                        .setLogBtnTextView("一键登录", 0xFFFFFFFF, 18)
                        .setLogBtnLoadingView(useCustomLoading ? "" : "umcsdk_load_dot_white", 20, 20, 12)
                        // .setNumberView(R.color.GOLColorNumberText, 24, 250, 0, numberLeft)
                        // .setSwitchView("切换账号", R.color.colorPrimary, 14, false, 250, 0, switchLeft)
                        // .setSwitchViewLayout("", switchWidth, 25)
                        .setPrivacyLayout(280, 400, 0, 0, true)
                        .setPrivacyCheckBox("gt_one_login_unchecked", "gt_one_login_checked", false, 9, 9, 2)
                        .setPrivacyClauseView(Color.parseColor("#80838A"), Color.parseColor("#FFC800"), 12)
                        .setPrivacyClauseViewTypeface(Typeface.defaultFromStyle(Typeface.BOLD), Typeface.defaultFromStyle(Typeface.NORMAL))
                        // 预留一组条款，SDK 填充运营商隐私条款
                        .setPrivacyClauseTextStrings("我已阅读并同意", "产品概述", "https://docs.geetest.com/onelogin/overview/prodes/", "",


                                "和", "Android 开发文档", "https://docs.geetest.com/onelogin/deploy/android", "",
                                "和", "常见问题", "https://docs.geetest.com/onelogin/help/faq", "",
                                "和", "", "", "并使用本机号码登录")
                        .setPrivacyAddFrenchQuotes(true)
                        .setProtocolShakeStyle(ProtocolShakeStyle.SHAKE_HORIZONTAL)
                        .setOneLoginAuthListener((context, oneLoginAuthCallback) -> {
                            // 创建 AlertDialog.Builder 对象
                            AlertDialog.Builder builder1 = new AlertDialog.Builder(context);
                            // 设置对话框标题
                            builder1.setTitle("自定义标题");
                            // 设置对话框信息
                            builder1.setMessage("自定义内容");
                            // 添加同意按钮及其处理逻辑
                            builder1.setPositiveButton("自定义同意", (dialogInterface, i) -> {
                                // 处理同意按钮逻辑
                                oneLoginAuthCallback.onOLAuthCallback(true);
                            });
                            // 添加拒绝按钮及其处理逻辑
                            builder1.setNegativeButton("自定义取消", (dialogInterface, i) -> {
                                // 处理拒绝按钮逻辑
                                dialogInterface.dismiss();
                            });
                            // 创建 AlertDialog 对象并返回
                            builder1.create();
                            builder1.show();
                        })
                        .build();
            case ONELOGIN_LANDSCAPE_MODE:
                return new OneLoginThemeConfig.Builder()
                        .setAuthBGImgPath("gt_one_login_bg")
                        .setDialogTheme(false, 300, 500, 0, 0, false, false)
                        .setStatusBar(0, UserInterfaceStyle.UNSPECIFIED, true)
                        .setAuthNavLayout(R.color.colorPrimary, 49, true, false)
                        .setAuthNavTextView("一键登录", 0xFFFFFFFF, 17, false, "服务条款", 0xFF000000, 17)
                        .setAuthNavReturnImgView("gt_one_login_ic_chevron_left_black", 45, 45, false, 8)
                        .setLogoImgView("gt_one_login_logo", 71, 71, false, 55, 0, 0)
                        .setSloganView(R.color.GOLColorGray, 10, 226, 0, 0)
                        .setNumberView(R.color.GOLColorNumberText, 24, 84, 0, 0)
                        .setSwitchView("切换账号", R.color.colorPrimary, 14, false, 128, 0, 0)
                        .setLogBtnLayout("gt_one_login_btn", "gt_one_login_btn_unchecked",268, 36, 169, 0, 0)
                        .setLogBtnTextView("一键登录", 0xFFFFFFFF, 15)
                        .setLogBtnLoadingView(useCustomLoading ? "" : "umcsdk_load_dot_white", 20, 20, 12)
                        .setPrivacyCheckBox("gt_one_login_unchecked", "gt_one_login_checked", false, 9, 9)
                        .setPrivacyLayout(268, 0, 1, 0, true)
                        .setPrivacyClauseView(R.color.GOLColorGray, R.color.colorPrimary, 10)
                        .setPrivacyTextView("登录即同意", "和", "、", "并使用本机号码登录")
                        .setPrivacyClauseText("用户协议", "https://docs.geetest.com/onelogin/deploy/android",
                                "隐私政策", "https://docs.geetest.com/onelogin/deploy/android",
                                null, null
                        )
                        .build();
        }
        return new OneLoginThemeConfig.Builder()
                .setLogBtnLoadingView("loading_icon", 20, 20, 12)
                .setAuthNavReturnImgView("gt_one_login_ic_chevron_left_black", 40, 40, false, 8)
                .build();
    }

    /**
     * 获得NavigationBar的高度
     */
    public int getNavigationBarHeight() {
        int result = 0;
        Resources resources = getResources();
        int resourceId = resources.getIdentifier("navigation_bar_height", "dimen", "android");
        if (resourceId > 0 && checkHasNavigationBar()) {
            result = resources.getDimensionPixelSize(resourceId);
        }
        return result;
    }

    /**
     * 判断是否有NavigationBar
     */
    public boolean checkHasNavigationBar() {
        WindowManager windowManager = getActivity().getWindowManager();
        Display d = windowManager.getDefaultDisplay();

        DisplayMetrics realDisplayMetrics = new DisplayMetrics();
        d.getRealMetrics(realDisplayMetrics);

        int realHeight = realDisplayMetrics.heightPixels;
        int realWidth = realDisplayMetrics.widthPixels;

        DisplayMetrics displayMetrics = new DisplayMetrics();
        d.getMetrics(displayMetrics);

        int displayHeight = displayMetrics.heightPixels;
        int displayWidth = displayMetrics.widthPixels;

        return (realWidth - displayWidth) > 0 || (realHeight - displayHeight) > 0;
    }
}