package com.geetest.gateauth.demo.ui.onepass;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

import com.geetest.gateauth.demo.SuccessActivity;
import com.geetest.gateauth.demo.util.Constants;
import com.geetest.gateauth.demo.R;
import com.geetest.gateauth.demo.util.HttpUtils;
import com.geetest.gateauth.demo.util.Utils;
import com.geetest.onelogin.view.LoadingImageView;
import com.geetest.onepassv2.OnePassHelper;
import com.geetest.onepassv2.listener.OnePassListener;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

public class OnePassFragment extends Fragment implements View.OnClickListener {

    private EditText numberEdt;
    private LoadingImageView loadingImageView;
    private String mNumber;
    private OnePassListener onePassListener;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_onepass, container, false);
        numberEdt = root.findViewById(R.id.gt_one_pass_number_input);
        Button submitBtn = root.findViewById(R.id.gt_one_pass_submit_tv);
        submitBtn.setOnClickListener(this);
        loadingImageView = root.findViewById(R.id.gt_one_pass_submit_iv);
        loadingImageView.bringToFront();
        return root;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initData();
    }

    private void initData() {
        mNumber = OnePassHelper.with().getCachedNumber();
        if (!TextUtils.isEmpty(mNumber)) {
            numberEdt.setText(mNumber);
        }
        initGop();
    }

    private void initGop() {
        OnePassHelper.with().init(this.getActivity(), Constants.APP_ID_OP, 8000);
        onePassListener = new OnePassListener() {

            @Override
            public void onTokenFail(JSONObject jsonObject) {
                Log.i(Constants.TAG, "onTokenFail:" + jsonObject.toString());
                Utils.toastMessage(getContext(), "本机认证失败");
                stopLoading();
            }

            @Override
            public void onTokenSuccess(final JSONObject jsonObject) {
                Log.i(Constants.TAG, "onTokenSuccess:" + jsonObject.toString());
                try {
                    jsonObject.put("id_2_sign", Constants.APP_ID_OP);
                    jsonObject.put("phone", mNumber);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                Log.i(Constants.TAG, "开始请求 checkgateway:" + jsonObject.toString());
                new CheckGatewayTask(OnePassFragment.this, jsonObject).execute(Constants.CHECK_GATE_WAY);
            }
        };
    }

    private void handleLogin(String phone) {
        if (TextUtils.isEmpty(phone) || !Utils.checkPhoneNum(phone)) {
            return;
        }
        mNumber = phone;
        //检测手机号格式
        if (Utils.checkPhoneNum(mNumber)) {
            if (!Utils.haveIntent(getContext())) {
                AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
                builder.setMessage("检测到未开启移动数据，建议手动开启进行操作");
                builder.setTitle("提示");
                builder.setPositiveButton("好的，我这就去开启", (dialog, which) -> {
                    dialog.dismiss();
                });
                builder.setNegativeButton("不好，我就不开", (dialog, which) -> {
                    startGOP();
                });
                builder.create().show();
            } else {
                startGOP();
            }
        } else {
            Utils.toastMessage(getContext(), "输入手机号格式不正确");
        }
    }

    private void stopLoading() {
        if (loadingImageView != null) {
            loadingImageView.play();
        }
    }

    @Override
    public void onClick(View v) {
        handleLogin(numberEdt.getText().toString().trim());
    }

    private void startGOP() {
        startLoading();
        OnePassHelper.with().getToken(mNumber, onePassListener);
    }

    private void startLoading() {
        if (loadingImageView != null) {
            loadingImageView.stop();
        }
    }

    static class CheckGatewayTask extends AsyncTask<String, Void, String> {
        /**
         * 构造网关校验参数
         */
        private final JSONObject jsonObject;
        private final WeakReference<OnePassFragment> fragment;

        public CheckGatewayTask(OnePassFragment fragment, JSONObject jsonObject) {
            this.fragment = new WeakReference<>(fragment);
            this.jsonObject = jsonObject;
        }

        @Override
        protected String doInBackground(String... params) {
            if (isCancelled()) {
                return null;
            }
            return HttpUtils.requestNetwork(params[0], jsonObject, null);
        }

        @Override
        protected void onPostExecute(String s) {
            Log.i(Constants.TAG, "CheckGateway 请求结束");
            if (isCancelled()) {
                return;
            }
            if (TextUtils.isEmpty(s)) {
                Log.e(Constants.TAG, "CheckGateway 校验异常");
                handleFailed(fragment);
                return;
            }
            Log.i(Constants.TAG, "CheckGateway 请求成功:" + s);
            fragment.get().stopLoading();
            try {
                JSONObject jsonObject = new JSONObject(s);
                int status = jsonObject.getInt("status");
                if (status == Constants.SUCCESS_CODE && "0".equals(jsonObject.getString("result"))) {
                    Intent intent = new Intent(fragment.get().getContext(), SuccessActivity.class);
                    intent.putExtra(Constants.IS_ONELOGIN_SUCCEED, true);
                    fragment.get().getContext().startActivity(intent);
                } else {
                    Log.e(Constants.TAG, "CheckGateway 校验失败:" + s);
                    handleFailed(fragment);
                }
            } catch (Exception e) {
                Log.e(Constants.TAG, "CheckGateway 校验失败:" + s);
                handleFailed(fragment);
            }
        }

        private void handleFailed(WeakReference<OnePassFragment> fragment) {
            try {
                Utils.toastMessage(fragment.get().getContext(), "本机认证校验失败");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}